﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Sample.Models;
using Infragistics.Web.Mvc;
using Core.Data;
using Core;
using Core.Data.BusinessObjects;

namespace Sample.Controllers
{
    public class ModelController : Controller
    {
        public ActionResult AutoGenerateColumns()
        {
            GridModel gridModel = new GridModel();
            gridModel.AutoGenerateColumns = true;
            gridModel.AutoGenerateLayouts = true;
            gridModel.DataSource = GetProjectData();
            gridModel.DataSourceUrl = Url.Action("BindAutoGeneratedProjects");
            gridModel.LoadOnDemand = false;

            ViewModel viewModel = new ViewModel();

            GridFiltering filtering = new GridFiltering();
            filtering.Type = OpType.Remote;

            gridModel.Features.Add(filtering);

            viewModel.Grid = gridModel;

            return View(viewModel);
        }

        public ActionResult DefinedColumns()
        {
            ViewModel viewModel = new ViewModel();

            viewModel.Grid = GetGridModel();

            return View(viewModel);
        }

        private GridModel GetGridModel()
        {
            GridModel gridModel = new GridModel();
            gridModel.AutoGenerateColumns = false;
            gridModel.PrimaryKey = "ProjectId";
            gridModel.DefaultColumnWidth = "200";
            gridModel.LoadOnDemand = false;

            gridModel.Columns = new List<GridColumn>();
            List<GridColumn> columns = new List<GridColumn>();
            columns.Add(new GridColumn("ProjectId", "ProjectId", "number", "200px"));
            columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            columns.Add(new GridColumn("StartDate", "StartDate", "date", "200px"));
            columns.Add(new GridColumn("EndDate", "EndDate", "date", "200px"));
            columns.Add(new GridColumn("Is Postponed", "IsPostponed", "bool", "200px"));
            gridModel.Columns = columns;

            GridFiltering filtering = new GridFiltering();
            filtering.Type = OpType.Remote;
            filtering.Inherit = true;

            GridPaging paging = new GridPaging();
            paging.Type = OpType.Remote;
            paging.PageSize = 2;
            paging.Inherit = true;

            gridModel.Features.Add(filtering);
            gridModel.Features.Add(paging);

            GridColumnLayoutModel vm = new GridColumnLayoutModel();
            vm = GetVirtualMachinesLayout();

            GridColumnLayoutModel dev = new GridColumnLayoutModel();
            dev = GetDevelopersLayout();
            dev.ColumnLayouts.Add(vm);

            GridColumnLayoutModel scrums = new GridColumnLayoutModel();
            scrums = GetScrumsLayout();

            GridColumnLayoutModel files = new GridColumnLayoutModel();
            files = GetFilesLayout();

            GridColumnLayoutModel checkins = new GridColumnLayoutModel();
            checkins = GetCheckinsLayout();
            checkins.ColumnLayouts.Add(files);

            GridColumnLayoutModel tasks = new GridColumnLayoutModel();
            tasks = GetTasksLayout();
            tasks.ColumnLayouts.Add(checkins);

            gridModel.ColumnLayouts.Add(dev);
            gridModel.ColumnLayouts.Add(scrums);
            gridModel.ColumnLayouts.Add(tasks);

            gridModel.DataSource = GetProjectData();
            gridModel.DataSourceUrl = Url.Action("BindProjects");


            return gridModel;
        }

        #region Data Access

        private IQueryable<MyProject> GetProjectData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from p in entities.Projects
                        orderby p.ProjectID
                        select new MyProject
                        {
                            ProjectId = p.ProjectID,
                            Name = p.Name,
                            StartDate = p.StartDate,
                            EndDate = p.EndDate,
                            IsPostponed = p.IsPostponed,
                            Developers = from d in p.Developers
                                         orderby d.DeveloperID
                                         select new MyDeveloper
                                         {
                                             DeveloperId = d.DeveloperID,
                                             ProjectId = d.ProjectID,
                                             Name = d.Name,
                                             Position = d.Position,
                                             HireDate = d.HireDate,
                                             Salary = d.Salary,
                                             VirtualMachines = from v in d.VirtualMachines
                                                               orderby v.VirtualMachineID
                                                               select new MyVirtualMachine
                                                               {
                                                                   VirtualMachineId = v.VirtualMachineID,
                                                                   DeveloperId = v.DeveloperID,
                                                                   OperatingSystem = v.OperatingSystem,
                                                                   DelegatedSpace = v.DelegatedSpace,
                                                                   ExpirationDate = v.ExpirationDate
                                                               }
                                         },
                            Scrums = from s in p.Scrums
                                     orderby s.MeetingID
                                     select new MyScrum
                                     {
                                         ProjectId = s.ProjectID,
                                         MeetingId = s.MeetingID,
                                         OccurredOn = s.OccurredOn,
                                         ScrumMaster = s.ScrumMaster,
                                         Notes = s.Notes,
                                         Priority = s.Priority,
                                         AreAllRequired = s.AreAllRequired,
                                         ScrumMeetings = new MyScrumMeeting()
                                         {
                                             MeetingId = s.ScrumMeeting.MeetingID,
                                             Name = s.ScrumMeeting.Name,
                                             Description = s.ScrumMeeting.Description,
                                             MaxDuration = s.ScrumMeeting.MaxDuration
                                         }
                                     },
                            Tasks = from t in p.Tasks
                                    orderby t.TaskID
                                    select new MyTask
                                    {
                                        TaskId = t.TaskID,
                                        ProjectId = t.ProjectID,
                                        Name = t.Name,
                                        ParentTaskId = t.ParentTaskID,
                                        IsCompleted = t.IsCompleted,
                                        Checkins = from c in t.Checkins
                                                   orderby c.CheckinID
                                                   select new MyCheckin
                                                   {
                                                       CheckinId = c.CheckinID,
                                                       TaskId = c.TaskID,
                                                       CheckinDate = c.CheckinDate,
                                                       Files = from f in c.Files
                                                               orderby f.FileID
                                                               select new MyFile
                                                               {
                                                                   FileId = f.FileID,
                                                                   CheckinId = f.CheckinID,
                                                                   Name = f.Name,
                                                                   Size = f.Size
                                                               }
                                                   }
                                    }
                        };
            return query;
        }

        private IEnumerable<MyDeveloper> GetDevelopersData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from d in entities.Developers
                        orderby d.DeveloperID
                        select new MyDeveloper
                        {
                            DeveloperId = d.DeveloperID,
                            ProjectId = d.ProjectID,
                            Name = d.Name,
                            Position = d.Position,
                            HireDate = d.HireDate,
                            Salary = d.Salary,
                            VirtualMachines = from v in d.VirtualMachines
                                              orderby v.VirtualMachineID
                                              select new MyVirtualMachine
                                              {
                                                  VirtualMachineId = v.VirtualMachineID,
                                                  DeveloperId = v.DeveloperID,
                                                  OperatingSystem = v.OperatingSystem,
                                                  DelegatedSpace = v.DelegatedSpace,
                                                  ExpirationDate = v.ExpirationDate
                                              }
                        };
            return query;
        }

        private IEnumerable<MyVirtualMachine> GetVirtualMachinesData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from v in entities.VirtualMachines
                        orderby v.VirtualMachineID
                        select new MyVirtualMachine
                        {
                            VirtualMachineId = v.VirtualMachineID,
                            DeveloperId = v.DeveloperID,
                            OperatingSystem = v.OperatingSystem,
                            DelegatedSpace = v.DelegatedSpace,
                            ExpirationDate = v.ExpirationDate
                        };
            return query;
        }

        private IEnumerable<MyScrum> GetScrumsData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from s in entities.Scrums
                        orderby s.MeetingID
                        select new MyScrum
                        {
                            ProjectId = s.ProjectID,
                            MeetingId = s.MeetingID,
                            OccurredOn = s.OccurredOn,
                            ScrumMaster = s.ScrumMaster,
                            Notes = s.Notes,
                            Priority = s.Priority,
                            AreAllRequired = s.AreAllRequired,
                            ScrumMeetings = new MyScrumMeeting()
                                         {
                                             MeetingId = s.ScrumMeeting.MeetingID,
                                             Name = s.ScrumMeeting.Name,
                                             Description = s.ScrumMeeting.Description,
                                             MaxDuration = s.ScrumMeeting.MaxDuration
                                         }
                        };
            return query;
        }

        private IEnumerable<MyTask> GetTasksData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from t in entities.Tasks
                        orderby t.TaskID
                        select new MyTask
                        {
                            TaskId = t.TaskID,
                            ProjectId = t.ProjectID,
                            Name = t.Name,
                            ParentTaskId = t.ParentTaskID,
                            IsCompleted = t.IsCompleted,
                            Checkins = from c in t.Checkins
                                       orderby c.CheckinID
                                       select new MyCheckin
                                       {
                                           CheckinId = c.CheckinID,
                                           TaskId = c.TaskID,
                                           CheckinDate = c.CheckinDate,
                                           Files = from f in c.Files
                                                   orderby f.FileID
                                                   select new MyFile
                                                   {
                                                       FileId = f.FileID,
                                                       CheckinId = f.CheckinID,
                                                       Name = f.Name,
                                                       Size = f.Size
                                                   }
                                       }
                        };
            return query;
        }

        private IEnumerable<MyCheckin> GetCheckinsData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from c in entities.Checkins
                        orderby c.CheckinID
                        select new MyCheckin
                        {
                            CheckinId = c.CheckinID,
                            TaskId = c.TaskID,
                            CheckinDate = c.CheckinDate
                        };
            return query;
        }

        private IEnumerable<MyFile> GetFilesData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();

            var query = from f in entities.Files
                        orderby f.FileID
                        select new MyFile
                        {
                            FileId = f.FileID,
                            CheckinId = f.CheckinID,
                            Name = f.Name,
                            Size = f.Size
                        };
            return query;
        }

        #endregion // Data Access

        #region DataSourceUrls

        public JsonResult BindAutoGeneratedProjects()
        {
            GridModel gridModel = new GridModel();
            gridModel.AutoGenerateColumns = true;
            gridModel.AutoGenerateLayouts = true;
            gridModel.DataSource = GetProjectData();
            gridModel.DataSourceUrl = Url.Action("BindAutoGeneratedProjects");
            gridModel.LoadOnDemand = false;

            ViewModel viewModel = new ViewModel();

            GridSorting rootSorting = new GridSorting();
            rootSorting.Type = OpType.Remote;
            rootSorting.Mode = SortingMode.Multiple;
            rootSorting.Inherit = false;
            gridModel.Features.Add(rootSorting);

            GridPaging rootPaging = new GridPaging();
            rootPaging.Type = OpType.Remote;
            rootPaging.Inherit = false;
            gridModel.Features.Add(rootPaging);

            viewModel.Grid = gridModel;
            return gridModel.GetData();
        }

        public JsonResult BindProjects()
        {
            GridModel gridModel = GetGridModel();
            return gridModel.GetData();
        }

        public JsonResult BindDevelopers(string path, string layout)
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetDevelopersData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindScrums(string path, string layout)
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetScrumsData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindTasks(string path, string layout)
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetTasksData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindCheckins(string path, string layout)
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetCheckinsData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindFiles(string path, string layout)
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetFilesData();
            return gridModel.GetData(path, layout);
        }

        public JsonResult BindVirtualMachines(string path, string layout)
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetVirtualMachinesData();
            return gridModel.GetData(path, layout);
        }

        #endregion // DataSourceUrls

        #region Layouts

        private GridColumnLayoutModel GetDevelopersLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Developers";
            layout.PrimaryKey = "DeveloperId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Developers";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Developer Id", "DeveloperId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Positon", "Position", "string", "200px"));
            layout.Columns.Add(new GridColumn("Hire Date", "HireDate", "date", "200px"));
            layout.Columns.Add(new GridColumn("Salary", "Salary", "number", "200px"));

            layout.DataSourceUrl = Url.Action("BindDevelopers");

            return layout;
        }

        private GridColumnLayoutModel GetVirtualMachinesLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "VirtualMachines";
            layout.PrimaryKey = "VirtualMachineId";
            layout.ForeignKey = "DeveloperId";
            layout.Caption = "Virtual Machines";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Virtual Machine Id", "VirtualMachineId", "string", "200px"));
            layout.Columns.Add(new GridColumn("Developer Id", "DeveloperId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Operating System", "OperatingSystem", "string", "200px"));
            layout.Columns.Add(new GridColumn("Delegated Space", "DelegatedSpace", "number", "200px"));
            layout.Columns.Add(new GridColumn("Expiration Date", "ExpirationDate", "date", "200px"));

            layout.DataSourceUrl = Url.Action("BindVirtualMachines");

            return layout;
        }

        private GridColumnLayoutModel GetScrumsLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Scrums";
            layout.PrimaryKey = "MeetingId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Scrums";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Meeting Id", "MeetingId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Occurred On", "OccurredOn", "date", "200px"));
            layout.Columns.Add(new GridColumn("Scrum Master", "ScrumMaster", "string", "200px"));
            layout.Columns.Add(new GridColumn("Notes", "Notes", "string", "200px"));
            layout.Columns.Add(new GridColumn("Priority", "Priority", "number", "200px"));
            layout.Columns.Add(new GridColumn("Are All Required", "AreAllRequired", "bool", "200px"));

            layout.DataSourceUrl = Url.Action("BindScrums");

            return layout;
        }

        private GridColumnLayoutModel GetTasksLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Tasks";
            layout.PrimaryKey = "TaskId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Tasks";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Task Id", "TaskId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Parent Task Id", "ParentTaskId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Is Completed", "IsCompleted", "bool", "200px"));

            layout.DataSourceUrl = Url.Action("BindTasks");

            return layout;
        }

        private GridColumnLayoutModel GetCheckinsLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Checkins";
            layout.PrimaryKey = "CheckinId";
            layout.ForeignKey = "TaskId";
            layout.Caption = "CheckIns";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Checkin Id", "CheckinId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Task Id", "TaskId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Checkin Date", "CheckinDate", "date", "200px"));

            layout.DataSourceUrl = Url.Action("BindCheckins");

            return layout;
        }

        private GridColumnLayoutModel GetFilesLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Files";
            layout.PrimaryKey = "FileId";
            layout.ForeignKey = "CheckinId";
            layout.Caption = "Files";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Files Id", "FileId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Checkin Id", "CheckinId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Size", "Size", "number", "200px"));

            layout.DataSourceUrl = Url.Action("BindFiles");

            return layout;
        }

        #endregion // Layouts
    }
}
